# uafswrapper.tcl is easy to use wrapper script for UAFS.
# It helps to enable internal flood protection for other scripts without
# almost any source scripts modifying.
# See install and usage section below.
#
# version:1.0 by 
# CoolCold <coolcold at coolcold.org> 
# #rea@irc.coolcold.org:6667
#
#  __    __  __    _           _    
# \  \  /  /|  \  | |         | |   
#  \  \/  / |   \ | |  ____  _| |_  
#   \    /  | |\ \| | /  _ \|_   _| 
#   /    \  | | \   | | /_\/  | |   
#  /  /\  \ | |  \  | | \__   | |__ 
# /__/  \__\|_|   \_| \____/  \___/ 
#
# http://xnet.net.ru irc://xnet.net.ru
#
#
#
#		:::[  T h e   R u s s i a n   E g g d r o p  R e s o u r c e  ]::: 
#      ____                __                                                      
#     / __/___ _ ___ _ ___/ /____ ___   ___      ___   ____ ___ _     ____ __ __   
#    / _/ / _ `// _ `// _  // __// _ \ / _ \    / _ \ / __// _ `/    / __// // /   
#   /___/ \_, / \_, / \_,_//_/   \___// .__/ __ \___//_/   \_, / __ /_/   \___/    
#        /___/ /___/                 /_/    /_/           /___/ /_/                
#
#
#
# Created with FAR ( http://www.farmanager.com )
# and colorer ( http://colorer.sf.net )
#
# $Id: uafswrapper.tcl v 0.1 25.10.2004 7.40 PM CoolCold Exp $
#
# Requirements:
# eggdrop (windrop) 1.6.x (others should work too )
# tcl 8.3.x and 8.4.x ( others not tested )
# UAFS pre beta 4 or older (http://web.coolcold.org/scripts/uafs/)


##############
# Installing # 
#
# add source scripts/uafswrapper.tcl to ur eggdrop.conf,but after
# source scripts/uafs.tcl 
##############

##############
#    Usage   #
#
# ::uafswrapper::addbind "bind command here",for example
# ::uafswrapper::addbind "bind pub - \"!test\" proc_test"
# or
# set bindcmd {bind pub - "!test" proc_test}
# ::uafswrapper::addbind $bindcmd
# 2nd form allows to prevent using backslashes
#
# see uafswrapperdemo.tcl and readme for more info
#
##############

namespace eval uafswrapper {
	#defining debug mode,set to 0 to disable

	set debugmode "1"
	
	#unsetting default values for prevent mixin on startup
	catch {array unset defaults}
	#setting new default values
	array set defaults {
		flood "4:30"
		ignores "f|f"
		strict "0"
		hostmask "4"
		callback ""
	}
    
    #unsetting ufh handlers array
    catch {array unset handlers}

    ##############################
    # do not edit anything below #
    ##############################

	set ver "0.1"
	set numver "000100"
	set needuafs "000004"
	set info "UAFS wrapper"
	set infoline "$info version $ver by CoolCold <coolcold at coolcold.org>"

	
	set uafsloaded "0"

	putlog ""
	putlog "--- Loading UAFS wrapper script ---"
	putlog "Checking if UAFS is installed and has proper version..."    
    if {[info exists "::uafs::numver"]} {
    	putlog "Variable ::uafs::numver found,value is $::uafs::numver"
    	if {$::uafs::numver>=$needuafs} {
    		set uafsloaded "1"
    		putlog "UAFS version is okay,continue loading"
    	} else {
    		putlog "UAFS version \"$::uafs::numver\"is too old,we need at least \"$needuafs\""
    	}
    } else {
    	putlog "--------------------------------------------------"
    	putlog "| No UAFS.tcl installed or too old version found |"
    	putlog "| Install UAFS.tcl first and load it before      |"
    	putlog "| loading this script!                           |"
		putlog "--------------------------------------------------"
    }

}

proc ::uafswrapper::addbind { bindcmd {flood ""} {hostmask ""} {ignores ""} {strict ""} {callback ""}} {
variable uafsloaded;variable handlers;variable defaults;variable debugmode
	set result 1
	if {$debugmode=="1"} {putlog "--- UAFS Wrapper debug: addbind called for \"$bindcmd\""}
	if {$uafsloaded=="1"} {
		#uafs.tcl script loaded,so let's call registerhandle func
		if {$flood==""} {set flood $defaults(flood)}
		if {$ignores==""} {set ignores $defaults(ignores)}
		if {$strict==""} {set strict $defaults(strict)}
		if {$callback==""} {set callback $defaults(callback)}
		if {$hostmask==""} {set hostmask $defaults(hostmask)}
		set ufh [::uafs::registerhandle $flood $hostmask $ignores $callback $strict]
		if {$ufh>0} {
			#flood handler indentified successfully
        	
        	#let's mark original procedure name
        	set procname [lindex [split $bindcmd ] end]
        	foreach i [lrange [split $bindcmd ] 0 end-1] {
        		lappend bla $i
        	}
        	set bla [join $bla]
            
            ########################################
            #                                      #
            # Achtung! Graceful eggdrop hack based #
            # upon tcl's argument providing method #
            #                                      #
            ########################################

            lappend bla [list "::uafswrapper::genproc" "$procname" "$ufh"]
            if {$debugmode=="1"} {putlog "--- UAFS Wrapper debug: using \"$bla\" to set bind"}
            if {[catch {eval $bla} errdata]} {
            	#some error occured while setting bind,what ta fuck?
            	#btw,user may specify any tcl command,not bind and it
            	#will be evaluated ;)
            	set result "-1"
            	
            	if {$debugmode=="1"} {
                	putlog "--- UAFS Wrapper debug: Adding bind with uafs wrapper failed."
                	set errdata [split $errdata \n]
                	foreach i $errdata {
                		putlog "--- UAFS Wrapper debug: $i"
                	}
            	}
            }
            
		} else {
			set result "-1"
			if {$debugmode=="1"} {
				putlog "--- UAFS Wrapper debug: Adding bind with uafs wrapper failed."
				putlog "--- UAFS Wrapper debug: Registerhandle failed,return code is \"$ufh\""
				putlog "--- UAFS Wrapper debug: Error description:\"[::uafs::formatmessage $ufh]\""
			}
		}
	} else {
		set result "-1"
	}
return $result
}

proc ::uafswrapper::genproc { procname ufh {param1 ""} {param2 ""} {param3 ""} {param4 ""} {param5 ""} {param6 ""}} {
variable uafsloaded;variable handlers;variable defaults;variable debugmode

if {$uafsloaded} {
	set bla [::uafs::isflood $ufh $param1 $param2 $param4]
	switch -exact -- $bla\
	"1" {
		#flood detected
		if {$debugmode=="1"} {
			putlog "--- UAFS Wrapper debug: flood detected from $param1!$param2 on $param4"
		}

	}\
	"0" {
		#no flood detected
		if {$debugmode=="1"} {
			putlog "--- UAFS Wrapper debug: no flood detected from $param1!$param2 on $param4,calling original proc $procname"
		}
		if {[info commands $procname]=="$procname"} {
			set procargs [info args "$procname"]
			if {[llength $procargs]>6 || [llength $procargs]<1} {
				if {$debugmode=="1"} {
					putlog "--- UAFS Wrapper debug: argh...this check failed...what the fuck,this should never happen..."
					putlog "--- UAFS Wrapper debug: procedure \"$procname\" has [llength $procargs] params,this is no-no! Wrapper,and even more eggdrop supports from 1 to 6,not less or more!"
				}
			} else {
    			set c "0"
    			foreach i $procargs {
    				incr c
    				eval lappend pau $[concat param$c]
    			}
    			eval "$procname $pau"
			}
		}
	}\
	default {
		if {$debugmode=="1"} {
			putlog "--- UAFS Wrapper debug: flood check failed for $param1!$param2 on $param4,ufh is $ufh"
			putlog "--- UAFS Wrapper debug: Error description: \"[::uafs::formatmessage $bla]\""
		}
	}
} else {
   	if {$debugmode=="1"} {
   		putlog "--- UAFS Wrapper debug: no UAFS script found but wrapper's general procedure was called with params:"
		putlog "--- UAFS Wrapper debug: procname:\"$procname\" ufh:\"$ufh\" $param1 $param2 $param3 $param4 $param5 $param6"
   	}
	
}

}

putlog "--- $::uafswrapper::infoline loaded ---"